-- NyxGUI Library
-- by Jadya EU - Well of Eternity

-- Theme info:
 -- Dropdown
  -- l2_texture			backdrop texture
  -- l2_color           backdrop color
  -- l2_border			backdrop border
--

-- Vars
 -- Dropdown
  -- .list				the list of elements to show
  -- .index			    the selected element's index
  -- .OnValueChanged	function (don't use SetScript, just attach this var or hook the existing script)

if NyxGUIPass then return end

local ng = NyxGUI(NyxGUImajor)

ng:RegisterType("Dropdown")
ng:RegisterType("Dropdown2")

local mf

-- Dropdown
local function hideList()
 if ng.opened_dropdown then
  if ng.opened_dropdown.original_border then
   ng.opened_dropdown:SetBackdropBorderColor(ng.hex2rgba(ng.opened_dropdown.original_border))
  else
   ng.opened_dropdown:SetBackdropBorderColor({0, 0, 0, 0})
  end
  ng.opened_dropdown = nil
 end
 mf:Hide()
end

local function showList(self, isRefresh)
 if not self.list or (not isRefresh and mf.list == self.list and mf:IsVisible()) then 
  hideList()
  return
 end

 mf:SetParent(self)
 mf:SetPoint("TOP", self, "BOTTOM")
 mf:SetFrameStrata("DIALOG")
 mf.list = self.list
 mf.update_pre = self.update_pre
 mf.update_post = self.update_post
 mf.update_button = self.update_button
 mf:update()

 if self.highlighted_border then
  self:SetBackdropBorderColor(ng.hex2rgba(self.highlighted_border))
 else
  self:SetBackdropBorderColor({1, 1, 1, 1})
 end
 
 ng.opened_dropdown = self
 mf:Show()
end

function ng.scripts.Dropdown.OnClick(self)
 showList(self)
end

local function dropdown_listbutton_onclick(btn)
 ng.opened_dropdown:SetText(btn.left_text:GetText())
 ng.opened_dropdown.index = btn.index

 if ng.opened_dropdown.OnValueChanged then
  ng.opened_dropdown:OnValueChanged()
 end

 hideList()
end

local function select_index(self, i)
 self.index = i
 self:SetText(self.list[i])
 if self.OnValueChanged then
  self:OnValueChanged()
 end
end

local function dropdown_init(self)
 if not self.list then return end
 
 self:Select(1)
end

function ng.constructors.Dropdown(addon, name, parent, theme)
 local f = CreateFrame("Button", name, parent)
 
 if not mf then
  mf = ng:New("NyxGUI", "Listbox", "NyxGUIDropdownList", f)
  mf:SetSize(200, 200)
  mf:Initialize("NyxGUIDropdownListButton")
  mf.hideSelection = true
  mf:SetBackdropColor(0, 0, 0, 1)
  for k,v in pairs(mf.buttons) do
   v.func = dropdown_listbutton_onclick
   v:SetBackdropBorderColor(0.1, 0.1, 0.1, 0.2)
  end
  mf:Hide()
 end

 local tex = f:CreateTexture()
 tex:SetSize(24, 24)
 tex:SetTexture("Interface\\Buttons\\UI-MicroStream-Yellow")
 tex:SetPoint("RIGHT", f, "RIGHT", -5, 0)
 f.arrow = tex

 f:SetSize(80, 20)
 f.text = ng:New(addon, "Label", nil, f)
 f.text:SetPoint("TOPLEFT", f, "TOPLEFT", 1, -1)
 f.text:SetPoint("BOTTOMRIGHT", f.arrow, "BOTTOMLEFT", -1, 1)
 f.SetText = ng.set_text_function

 f:SetScript("OnHide", hideList)
 f:SetHighlightTexture("Interface\\Buttons\\UI-Listbox-Highlight2", "ADD")
 f.backdrop_type = "l2"
 
 f.Initialize = dropdown_init
 f.Select = select_index
 return f
end

function ng.set_theme.Dropdown(self, th)
 
 self:SetBackdropColor(ng.hex2rgba(th["l2_color"]))
 self:GetHighlightTexture():SetVertexColor(ng.hex2rgba(th["highlight"]))

 local r, g, b, a = ng.hex2rgba(th["l2_border"])
 self:SetBackdropBorderColor(r, g, b, a)
 self.original_border = th["l2_border"]
 self.highlighted_border = ng.rgba2hex(r + 0.3, g + 0.3, b + 0.3, a + 0.3)
end


-- no listbox version -- nyi

local theme_data = {
                    -- textures and colors
				     --l0_texture     = "Interface\\Buttons\\GreyscaleRamp64",
                     l0_color       = "121212e6",
				     l0_border      = "1A1A1Ae6",
					 --l1_texture     = "Interface\\Buttons\\WHITE8X8",
					 --l1_color       = "000000FF",
					 --l1_border      = "1A1A1AFF",
					 --l2_texture     = "Interface\\Buttons\\WHITE8X8",
					 --l2_color       = "1A1A1AFF",
					 --l2_border      = "1A1A1AFF",
					 --l3_texture     = "Interface\\Buttons\\WHITE8X8",
					 l3_color       = "0A0A0AFF",
					 l3_border      = "0A0A0AFF",
					 --highlight      = "FFFFFF33",
					-- fonts
					 --f_label_name   = "Fonts\\FRIZQT__.ttf",
					 --f_label_h      = 10,
					 --f_label_flags  = "",
					 --f_label_color  = "FFFFFFFF",
					 --f_button_name  = "Fonts\\FRIZQT__.ttf",
					 --f_button_h     = 12,
					 --f_button_flags = "",
					 --f_button_color = "FFFFFFFF",
				   }


ng:SetThemeData("NyxGUI", "dropdown", theme_data)

local mf

local function hideList()
 if mf.opened_dd then
  if mf.opened_dd.original_border then
   mf.opened_dd:SetBackdropBorderColor(ng.hex2rgba(mf.opened_dd.original_border))
  else
   mf.opened_dd:SetBackdropBorderColor({0, 0, 0, 0})
  end
  mf.opened_dd = nil
 end
 mf:Hide()
end

local function showList(self, isRefresh)
 if not self.list or (not isRefresh and mf:IsVisible() and self == mf.opened_dd) then
  hideList()
  return
 end

 mf:SetParent(self)
 mf:SetPoint("TOP", self, "BOTTOM")
 mf:SetFrameStrata("DIALOG")
 mf.list = self.list
 mf.update_pre = self.update_pre
 mf.update_post = self.update_post
 mf.update_button = self.update_button
 mf:update()

 if self.highlighted_border then
  self:SetBackdropBorderColor(ng.hex2rgba(self.highlighted_border))
 else
  self:SetBackdropBorderColor({1, 1, 1, 1})
 end
 
 mf.opened_dd = self
 mf:Show()
end

local function create_button(i)
 local f = ng:New("NyxGUI", "Button", "NyxGUIDropdownListButton"..i, mf, "dropdown")
 
 mf.buttons[i] = f
end

local function base_update_button(btn, k, v)
 btn:SetText(k)
end

function ng.scripts.Dropdown2.OnClick(self)
 showList(self)
end

function ng.constructors.Dropdown2(addon, name, parent, theme)
 local f = CreateFrame("Button", name, parent)
 
 if not mf then
  mf = ng:New("NyxGUI", "Frame", "NyxGUIDropdownList", f, "dropdown")
  mf:SetSize(200, 200)
  mf:SetBackdropColor(0, 0, 0, 1)
  mf.buttons = {}
  mf.update_button = base_update_button
  mf.update = function(self)
               if not self.list then return end

			   local btn = 1
			   for k,v in pairs(self.list) do
			    if not self.buttons[btn] then create_button(btn) end
                self.update_button(self.buttons[btn], k, v)
				btn = btn + 1
			   end
              end
  mf:Hide()
 end

 local tex = f:CreateTexture()
 tex:SetSize(24, 24)
 tex:SetTexture("Interface\\Buttons\\UI-MicroStream-Yellow")
 tex:SetPoint("RIGHT", f, "RIGHT", -5, 0)
 f.arrow = tex

 f:SetSize(80, 20)
 f.text = ng:New(addon, "Label", nil, f)
 f.text:SetPoint("TOPLEFT", f, "TOPLEFT", 1, -1)
 f.text:SetPoint("BOTTOMRIGHT", f.arrow, "BOTTOMLEFT", -1, 1)
 f.SetText = ng.set_text_function

 f:SetScript("OnHide", hideList)
 f:SetHighlightTexture("Interface\\Buttons\\UI-Listbox-Highlight2", "ADD")
 f.backdrop_type = "l2"
 
 f.Initialize = dropdown_init
 f.Select = select_index
 return f
end

function ng.set_theme.Dropdown2(self, th)
 self:SetBackdropColor(ng.hex2rgba(th["l2_color"]))
 self:GetHighlightTexture():SetVertexColor(ng.hex2rgba(th["highlight"]))

 local r, g, b, a = ng.hex2rgba(th["l2_border"])
 self:SetBackdropBorderColor(r, g, b, a)
 self.original_border = th["l2_border"]
 self.highlighted_border = ng.rgba2hex(r + 0.3, g + 0.3, b + 0.3, a + 0.3)
end